(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('react-dom')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react', 'react-dom'], factory) :
    (global = global || self, factory(global.Floaty = {}, global.React, global.ReactDOM));
}(this, function (exports, React, ReactDOM) { 'use strict';

    var __rest = (undefined && undefined.__rest) || function (s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
            t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    };
    const styleElement = document.createElement('style');
    styleElement.innerHTML = '.react-managed-draggable-unselectable { user-select: none; }';
    document.body.appendChild(styleElement);
    const rafThrottle = (callback) => {
        let requestId;
        const later = (context, args) => () => {
            requestId = null;
            callback.apply(context, args);
        };
        const throttled = function (...args) {
            if ((requestId === null) || (requestId === undefined)) {
                requestId = window.requestAnimationFrame(later(this, args));
            }
        };
        throttled.cancel = () => requestId !== null && window.cancelAnimationFrame(requestId);
        return throttled;
    };
    function manhattanDistance(p1, p2) {
        return Math.abs(p1.x - p2.x) + Math.abs(p1.y - p2.y);
    }
    function getPosition(event) {
        if ('touches' in event) {
            if (event.touches.length >= 1) {
                return { x: event.touches[0].clientX, y: event.touches[0].clientY };
            }
            else {
                return { x: 0, y: 0 };
            }
        }
        else {
            return { x: event.clientX, y: event.clientY };
        }
    }
    class ElementDraggable {
        constructor(element, options, immediate) {
            this.down = false;
            this.dragging = false;
            this.start = { x: 0, y: 0 };
            this.current = { x: 0, y: 0 };
            this.handleDown = (event) => {
                if (!this.down && !this.dragging) {
                    this.down = true;
                    document.addEventListener('mousemove', this.handleMove);
                    document.addEventListener('touchmove', this.handleMove);
                    document.addEventListener('mouseup', this.handleUp);
                    document.addEventListener('touchend', this.handleUp);
                    document.body.classList.add('react-managed-draggable-unselectable');
                    this.current = this.start = getPosition(event);
                    if (this.getThreshold() <= 0) {
                        this.dragging = true;
                        if (this.options.onDragStart) {
                            this.options.onDragStart(event, this.generateDragInformation(this.current));
                        }
                    }
                    event.stopPropagation();
                    event.preventDefault();
                }
            };
            this.handleMove = rafThrottle((event) => {
                if (this.down || this.dragging) {
                    const last = this.current;
                    this.current = getPosition(event);
                    if (this.dragging) {
                        if (this.options.onDragMove) {
                            this.options.onDragMove(event, this.generateDragInformation(last));
                        }
                    }
                    else {
                        if (manhattanDistance(this.start, this.current) >= this.getThreshold()) {
                            this.dragging = true;
                            if (this.options.onDragStart) {
                                this.options.onDragStart(event, this.generateDragInformation(last));
                            }
                        }
                    }
                }
            });
            this.handleUp = (event) => {
                if (this.down || this.dragging) {
                    this.down = false;
                    document.body.classList.remove('react-managed-draggable-unselectable');
                    document.removeEventListener('mousemove', this.handleMove);
                    document.removeEventListener('touchmove', this.handleMove);
                    document.removeEventListener('mouseup', this.handleUp);
                    document.removeEventListener('touchend', this.handleUp);
                    if (this.dragging) {
                        this.dragging = false;
                        const last = this.current;
                        // If touch is released, there are no event coordinates
                        // this.current = getPosition(event);
                        if (this.options.onDragEnd) {
                            this.options.onDragEnd(event, this.generateDragInformation(last));
                        }
                    }
                    else if (this.down && this.options.onClick) {
                        if ('touches' in event || event.button === 0) {
                            const last = this.current;
                            if (!('touches' in event)) {
                                this.current = getPosition(event);
                            }
                            this.options.onClick(event, this.generateDragInformation(last));
                        }
                    }
                }
            };
            this.element = element;
            this.options = options;
            this.immediate = !!immediate;
            if (immediate) {
                this.dragging = true;
                document.addEventListener('mousemove', this.handleMove);
                document.addEventListener('touchmove', this.handleMove);
                document.addEventListener('mouseup', this.handleUp);
                document.addEventListener('touchend', this.handleUp);
            }
            else {
                element.addEventListener('mousedown', this.handleDown);
                element.addEventListener('touchstart', this.handleDown);
            }
        }
        destroy() {
            if (!this.immediate) {
                this.element.removeEventListener('mousedown', this.handleDown);
                this.element.removeEventListener('touchstart', this.handleDown);
            }
            if (this.down) {
                this.down = false;
                if (this.options.onDragEnd) {
                    this.options.onDragEnd(undefined, this.generateDragInformation(this.current));
                }
                document.removeEventListener('mousemove', this.handleMove);
                document.removeEventListener('touchmove', this.handleMove);
                document.removeEventListener('mouseup', this.handleUp);
                document.removeEventListener('touchend', this.handleUp);
            }
        }
        getThreshold() {
            return this.options.threshold === undefined ? 0 : this.options.threshold;
        }
        generateDragInformation(last) {
            return {
                current: this.current,
                start: this.start,
                last
            };
        }
    }
    class Draggable extends React.Component {
        constructor() {
            super(...arguments);
            this.element = null;
            this.draggable = null;
        }
        componentDidMount() {
            if (this.element !== null) {
                this.draggable = new ElementDraggable(this.element, {
                    threshold: this.props.threshold,
                    onDragStart: this.props.onDragStart,
                    onDragMove: this.props.onDragMove,
                    onDragEnd: this.props.onDragEnd,
                    onClick: this.props.onClick
                });
            }
        }
        componentWillUnmount() {
            if (this.draggable !== null) {
                this.draggable.destroy();
                this.draggable = null;
            }
        }
        render() {
            const _a = this.props, { children, threshold, onDragStart, onDragMove, onDragEnd, onClick } = _a, other = __rest(_a, ["children", "threshold", "onDragStart", "onDragMove", "onDragEnd", "onClick"]);
            return React.createElement("div", Object.assign({ ref: (ref) => this.element = ref }, other), children);
        }
    }

    const computeDropArea = (element) => {
        const clientRect = element.getBoundingClientRect();
        return {
            top: clientRect.top,
            left: clientRect.left,
            width: clientRect.width,
            height: clientRect.height
        };
    };
    const dropAreasEqual = (dropAreaA, dropAreaB) => {
        return dropAreaA.top === dropAreaB.top
            && dropAreaA.left === dropAreaB.left
            && dropAreaA.width === dropAreaB.width
            && dropAreaA.height === dropAreaB.height;
    };
    const pointInDropArea = (dropArea, point) => {
        return dropArea.top <= point.y
            && point.y <= dropArea.top + dropArea.height
            && dropArea.left <= point.x
            && point.x <= dropArea.left + dropArea.width;
    };

    class StaticUtilities {
        static traverseLayoutTree(layout, callback, order) {
            if (order === 'pre-order') {
                callback(layout);
            }
            if (layout.type !== 'stack') {
                for (const item of layout.items) {
                    StaticUtilities.traverseLayoutTree(item.child, callback, order);
                }
            }
            if (order === 'post-order') {
                callback(layout);
            }
        }
        static minimizeLayout(layout) {
            if (layout === null) {
                return null;
            }
            switch (layout.type) {
                case 'column':
                case 'row': {
                    const type = layout.type;
                    const items = [];
                    let changed = false;
                    for (const item of layout.items) {
                        const result = this.minimizeLayout(item.child);
                        if (result !== item.child) {
                            changed = true;
                        }
                        if (result) {
                            if (result.type === type) {
                                const childSumFractions = result.items.map((item) => item.fraction).reduce((sum, fraction) => sum + fraction);
                                const normalizedChildFractions = result.items.map((item) => item.fraction / childSumFractions);
                                for (let i = 0; i < result.items.length; i++) {
                                    items.push(Object.assign({}, result.items[i], { fraction: normalizedChildFractions[i] * item.fraction }));
                                }
                                changed = true;
                            }
                            else {
                                items.push(Object.assign({}, item, { child: result }));
                            }
                        }
                    }
                    if (items.length >= 2) {
                        if (changed) {
                            // Normalize fractions.
                            const fractionSum = items.reduce((sum, item) => sum + item.fraction, 0);
                            const normalizedItems = items.map((item) => {
                                return Object.assign({}, item, { fraction: item.fraction / fractionSum });
                            });
                            return Object.assign({}, layout, { items: normalizedItems });
                        }
                        else {
                            return layout;
                        }
                    }
                    else if (items.length === 1) {
                        return items[0].child;
                    }
                    else {
                        return null;
                    }
                }
                case 'stack': {
                    if (layout.items.length >= 1) {
                        return layout;
                    }
                    else {
                        return null;
                    }
                }
            }
        }
        static findStackItem(predicate, layout) {
            if (layout.type === 'stack') {
                for (const item of layout.items) {
                    if (predicate(item.item)) {
                        return item;
                    }
                }
            }
            else {
                for (const item of layout.items) {
                    const found = StaticUtilities.findStackItem(predicate, item.child);
                    if (found) {
                        return found;
                    }
                }
            }
            return null;
        }
        static findStack(stackItem, from) {
            if (from === null) {
                return null;
            }
            switch (from.type) {
                case 'column':
                case 'row': {
                    for (const item of from.items) {
                        const found = StaticUtilities.findStack(stackItem, item.child);
                        if (found) {
                            return found;
                        }
                    }
                    break;
                }
                case 'stack': {
                    for (const item of from.items) {
                        if (stackItem === item) {
                            return from;
                        }
                    }
                    break;
                }
            }
            return null;
        }
        static getContentResolutionSide(resolution, mousePosition) {
            if (mousePosition.x <= resolution.dropArea.left + resolution.dropArea.width * 0.2) {
                return 'left';
            }
            else if (mousePosition.x >= resolution.dropArea.left + resolution.dropArea.width * 0.8) {
                return 'right';
            }
            else if (mousePosition.y <= resolution.dropArea.top + resolution.dropArea.height * 0.5) {
                return 'top';
            }
            else {
                return 'bottom';
            }
        }
    }

    class Floaty extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.state = {
                currentMousePosition: null,
                dropResolutions: [],
                rootDropArea: null
            };
            this.portal = document.createElement('div');
            this.dropResolutions = new Map();
            this.eventTarget = null;
            this.raf = undefined;
            this.updateDropAreas = () => {
                const node = ReactDOM.findDOMNode(this);
                if (node instanceof Element) {
                    const dropArea = computeDropArea(node);
                    if (!this.state.rootDropArea || !dropAreasEqual(this.state.rootDropArea, dropArea)) {
                        this.setState({ rootDropArea: dropArea });
                    }
                }
                this.raf = window.requestAnimationFrame(this.updateDropAreas);
            };
            this.onRowOrColumnUpdateFractions = (rowOrColumn, index1, fraction1, index2, fraction2) => {
                const path = this.findPath(rowOrColumn, this.props.state.layout);
                if (!path) {
                    if (rowOrColumn.type === 'column') {
                        throw new Error('Column not found.');
                    }
                    else {
                        throw new Error('Row not found.');
                    }
                }
                const items = rowOrColumn.items.slice();
                items[index1] = Object.assign({}, items[index1], { fraction: fraction1 });
                items[index2] = Object.assign({}, items[index2], { fraction: fraction2 });
                const newRowOrColumn = Object.assign({}, rowOrColumn, { items });
                this.replaceInPath(newRowOrColumn, path);
                this.onLayoutChange(path[path.length - 1]);
            };
            this.registerDropResolutions = (key, dropResolutions) => {
                this.dropResolutions.set(key, dropResolutions);
                this.updateDropResolutions();
            };
            this.unregisterDropResolutions = (key) => {
                this.dropResolutions.delete(key);
                this.updateDropResolutions();
            };
            this.updateColumnFractions = this.onRowOrColumnUpdateFractions;
            this.updateRowFractions = this.onRowOrColumnUpdateFractions;
            this.activateStackItem = (stackItem) => {
                const stack = this.findStack(stackItem);
                if (!stack) {
                    throw new Error(`StackItem ${stackItem.key} not found.`);
                }
                const index = stack.items.indexOf(stackItem);
                const path = this.findPath(stack, this.props.state.layout);
                if (!path) {
                    throw new Error('Stack not found.');
                }
                const newStack = Object.assign({}, stack, { active: index });
                this.replaceInPath(newStack, path);
                this.onLayoutChange(path[path.length - 1]);
            };
            this.closeTab = (stackItem) => {
                const stack = this.findStack(stackItem);
                if (!stack) {
                    throw new Error(`StackItem ${stackItem.key} not found.`);
                }
                const index = stack.items.indexOf(stackItem);
                const path = this.findPath(stack, this.props.state.layout);
                if (!path) {
                    throw new Error('Stack not found.');
                }
                const items = stack.items.slice();
                items.splice(index, 1);
                const newStack = Object.assign({}, stack, { items, active: Math.min(items.length - 1, stack.active) });
                this.replaceInPath(newStack, path);
                this.onLayoutChange(path[path.length - 1]);
            };
            this.replaceItem = (stackItem, item) => {
                const stack = this.findStack(stackItem);
                if (!stack) {
                    throw new Error(`StackItem ${stackItem.key} not found.`);
                }
                const index = stack.items.indexOf(stackItem);
                const path = this.findPath(stack, this.props.state.layout);
                if (!path) {
                    throw new Error('Stack not found.');
                }
                const newItems = stack.items.slice();
                newItems[index].item = item;
                const newStack = Object.assign({}, stack, { items: newItems });
                this.replaceInPath(newStack, path);
                this.onLayoutChange(path[path.length - 1]);
            };
            this.handleMove = (event) => {
                if (this.props.state.floating) {
                    event.preventDefault();
                    const position = getPosition(event);
                    this.setState({ currentMousePosition: position });
                }
            };
            this.handleUp = (event) => {
                if (!this.props.state.floating) {
                    return;
                }
                const position = event instanceof MouseEvent
                    ? getPosition(event)
                    : this.state.currentMousePosition;
                if (!position) {
                    return;
                }
                const resolution = this.getCandidateDropResolution(position);
                if (!resolution) {
                    return;
                }
                let path;
                if (resolution.type === 'root') {
                    path = [{ type: 'stack', active: 0, items: [this.props.state.floating] }];
                }
                else {
                    path = this.findPath(resolution.stack, this.props.state.layout);
                    if (!path) {
                        throw new Error('Stack not found.');
                    }
                    if (resolution.type === 'container') {
                        const side = StaticUtilities.getContentResolutionSide(resolution, position);
                        const stackFloating = { type: 'stack', items: [this.props.state.floating], active: 0 };
                        const childFloating = { child: stackFloating, fraction: 0.5 };
                        const childOriginal = { child: resolution.stack, fraction: 0.5 };
                        let newLayout;
                        if (side === 'left') {
                            newLayout = { type: 'row', items: [childFloating, childOriginal] };
                        }
                        else if (side === 'right') {
                            newLayout = { type: 'row', items: [childOriginal, childFloating] };
                        }
                        else if (side === 'top') {
                            newLayout = { type: 'column', items: [childFloating, childOriginal] };
                        }
                        else {
                            newLayout = { type: 'column', items: [childOriginal, childFloating] };
                        }
                        this.replaceInPath(newLayout, path);
                    }
                    else {
                        const items = resolution.stack.items.slice();
                        items.splice(resolution.index, 0, this.props.state.floating);
                        const newStack = Object.assign({}, resolution.stack, { items, active: resolution.index });
                        this.replaceInPath(newStack, path);
                    }
                }
                const newState = {
                    layout: path[path.length - 1],
                    floating: null
                };
                this.updateState(newState);
                this.unregisterFloatHandlers();
            };
            this.startFloat = (stackItem, options) => {
                if (this.props.state.floating) {
                    return;
                }
                const stack = this.findStack(stackItem);
                let newState;
                if (stack) {
                    const index = stack.items.indexOf(stackItem);
                    const path = this.findPath(stack, this.props.state.layout);
                    if (!path) {
                        throw new Error('Stack not found.');
                    }
                    const items = stack.items.slice();
                    items.splice(index, 1);
                    const newStack = Object.assign({}, stack, { items, active: Math.min(items.length - 1, stack.active) });
                    this.replaceInPath(newStack, path);
                    newState = {
                        layout: path[path.length - 1],
                        floating: stack.items[index]
                    };
                }
                else {
                    newState = {
                        layout: this.props.state.layout,
                        floating: stackItem
                    };
                }
                // Update floating position.
                if ('initialPosition' in options) {
                    this.setState({ currentMousePosition: options.initialPosition });
                }
                else {
                    this.setState({ currentMousePosition: getPosition(options.event) });
                }
                // Update controlled state.
                this.updateState(newState);
                if ('eventTarget' in options) {
                    this.registerFloatHandlers(options.eventTarget);
                }
                else if ('event' in options) {
                    if (options.event instanceof TouchEvent && options.event.target && options.event.target instanceof HTMLElement) {
                        // Touch events should be re-registered with their event target, otherwise the touchmove and touchup events will not fire.
                        this.registerFloatHandlers(options.event.target);
                    }
                    else {
                        // Mouse events can happily be attached to the body.
                        this.registerFloatHandlers(document.body);
                    }
                }
                else {
                    this.registerFloatHandlers(document.body);
                }
            };
            this.getLayout = () => {
                return this.props.state.layout;
            };
            this.findStack = (stackItem) => {
                return StaticUtilities.findStack(stackItem, this.getLayout());
            };
        }
        componentDidMount() {
            this.updateDropAreas();
            if (this.props.state.floating) {
                this.registerFloatHandlers(document.body);
            }
            document.body.appendChild(this.portal);
        }
        componentWillUnmount() {
            if (this.raf !== undefined) {
                window.cancelAnimationFrame(this.raf);
            }
            this.unregisterFloatHandlers();
            document.body.removeChild(this.portal);
        }
        render() {
            return React.createElement(React.Fragment, null,
                this.renderLayout(),
                this.renderFloating());
        }
        renderLayout() {
            if (this.props.state.layout) {
                return React.createElement(this.props.renderers.layoutRenderer, { floatyRenderers: this.props.renderers, floatyManager: this, layout: this.props.state.layout });
            }
            else {
                return React.createElement("div", { style: { width: '100%', height: '100%' } });
            }
        }
        renderDropResolution() {
            if (!this.state.currentMousePosition) {
                return;
            }
            const resolution = this.getCandidateDropResolution(this.state.currentMousePosition);
            if (!resolution) {
                return;
            }
            if (resolution.type === 'container') {
                const side = StaticUtilities.getContentResolutionSide(resolution, this.state.currentMousePosition);
                const sideDropArea = {
                    left: side === 'right' ? resolution.dropArea.left + resolution.dropArea.width * 0.5 : resolution.dropArea.left,
                    top: side === 'bottom' ? resolution.dropArea.top + resolution.dropArea.height * 0.5 : resolution.dropArea.top,
                    width: side === 'left' || side === 'right' ? resolution.dropArea.width * 0.5 : resolution.dropArea.width,
                    height: side === 'top' || side === 'bottom' ? resolution.dropArea.height * 0.5 : resolution.dropArea.height
                };
                return React.createElement(this.props.renderers.dropAreaRenderer, { floatyManager: this, dropArea: sideDropArea });
            }
            else {
                return React.createElement(this.props.renderers.dropAreaRenderer, { floatyManager: this, dropArea: resolution.dropArea });
            }
        }
        renderFloating() {
            if (this.props.state.floating && this.state.currentMousePosition) {
                return ReactDOM.createPortal(React.createElement(React.Fragment, null,
                    this.renderDropResolution(),
                    React.createElement("div", { style: { position: 'fixed', top: this.state.currentMousePosition.y, left: this.state.currentMousePosition.x } },
                        React.createElement(this.props.renderers.floatingRenderer, { floatyRenderers: this.props.renderers, floatyManager: this, floating: this.props.state.floating }))), this.portal);
            }
        }
        getCandidateDropResolution(position) {
            if (this.props.state.floating) {
                for (const resolution of this.state.dropResolutions) {
                    if (pointInDropArea(resolution.dropArea, position)) {
                        return resolution;
                    }
                }
                if (this.state.rootDropArea && pointInDropArea(this.state.rootDropArea, position)) {
                    return { type: 'root', dropArea: this.state.rootDropArea };
                }
            }
            return null;
        }
        registerFloatHandlers(eventTarget) {
            if (this.eventTarget) {
                return;
            }
            eventTarget.addEventListener('mousemove', this.handleMove, { passive: false });
            eventTarget.addEventListener('touchmove', this.handleMove, { passive: false });
            eventTarget.addEventListener('mouseup', this.handleUp, { passive: false });
            eventTarget.addEventListener('touchend', this.handleUp, { passive: false });
            this.eventTarget = eventTarget;
        }
        unregisterFloatHandlers() {
            if (!this.eventTarget) {
                return;
            }
            this.eventTarget.removeEventListener('mousemove', this.handleMove);
            this.eventTarget.removeEventListener('touchmove', this.handleMove);
            this.eventTarget.removeEventListener('mouseup', this.handleUp);
            this.eventTarget.removeEventListener('touchend', this.handleUp);
            this.eventTarget = null;
        }
        updateState(state) {
            this.props.onStateChange(Object.assign({}, state, { layout: StaticUtilities.minimizeLayout(state.layout) }));
        }
        onLayoutChange(layout) {
            const newState = Object.assign({}, this.props.state, { layout });
            this.updateState(newState);
        }
        updateDropResolutions() {
            const dropResolutions = [];
            for (const dropResolution of this.dropResolutions.values()) {
                dropResolutions.push(...dropResolution);
            }
            this.setState({ dropResolutions });
        }
        replaceInPath(target, path) {
            let previous = path[0];
            path[0] = target;
            for (let i = 1; i < path.length; i++) {
                // The first item of the path can be a Stack, but the rest are always Columns and Rows.
                const node = path[i];
                const index = node.items.findIndex((item) => item.child === previous);
                const items = node.items.slice();
                items[index] = Object.assign({}, items[index], { child: path[i - 1] });
                previous = path[i];
                path[i] = Object.assign({}, node, { items });
            }
        }
        findPath(target, from) {
            if (from === null) {
                return null;
            }
            if (target === from) {
                return [target];
            }
            switch (from.type) {
                case 'column':
                case 'row': {
                    for (const item of from.items) {
                        const found = this.findPath(target, item.child);
                        if (found) {
                            return [...found, from];
                        }
                    }
                    break;
                }
            }
            return null;
        }
    }

    class ColumnRenderer extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.container = null;
            this.handleRef = (ref) => {
                this.container = ref;
            };
            this.clamp = (index, deltaY) => {
                if (this.container) {
                    const heightA = this.container.children[index * 2].getBoundingClientRect().height;
                    const heightB = this.container.children[index * 2 + 2].getBoundingClientRect().height;
                    return Math.min(heightB, Math.max(-heightA, deltaY));
                }
                return Number.NaN;
            };
            this.handleMove = (index, deltaY) => {
                if (this.container) {
                    const heightA = this.container.children[index * 2].getBoundingClientRect().height;
                    const heightS = this.container.children[index * 2 + 1].getBoundingClientRect().height;
                    const heightB = this.container.children[index * 2 + 2].getBoundingClientRect().height;
                    const heightTotal = heightA + heightS + heightB;
                    const fractionA = heightA / heightTotal;
                    const fractionS = heightS / heightTotal;
                    const fractionB = heightB / heightTotal;
                    const fractionTotal = fractionA + fractionS + fractionB;
                    const deltaF = fractionTotal * deltaY / heightTotal;
                    const fractionOriginal = this.props.column.items[index].fraction + this.props.column.items[index + 1].fraction;
                    const normalizer = fractionOriginal / (fractionA + fractionB);
                    this.props.floatyManager.updateColumnFractions(this.props.column, index, normalizer * (fractionA + deltaF), index + 1, normalizer * (fractionB - deltaF));
                }
            };
        }
        render() {
            const gridTemplateRows = [];
            const elements = [];
            for (let i = 0; i < this.props.column.items.length; i++) {
                if (i > 0) {
                    gridTemplateRows.push('max-content');
                    elements.push(React.createElement(this.props.floatyRenderers.columnSeparatorRenderer, { key: `${i}-seperator`, floatyRenderers: this.props.floatyRenderers, index: i - 1, onMove: this.handleMove, clamp: this.clamp }));
                }
                gridTemplateRows.push(`minmax(0, ${this.props.column.items[i].fraction}fr)`);
                elements.push(React.createElement(this.props.floatyRenderers.layoutRenderer, { key: i, floatyRenderers: this.props.floatyRenderers, floatyManager: this.props.floatyManager, layout: this.props.column.items[i].child }));
            }
            return React.createElement("div", { ref: this.handleRef, style: { display: 'grid', gridTemplateRows: gridTemplateRows.join(' '), width: '100%', height: '100%' } }, elements);
        }
    }

    class ColumnSeparatorHandleRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: { position: 'relative', width: '100%', height: 6 } },
                React.createElement("div", { style: {
                        backgroundColor: 'black',
                        position: 'absolute',
                        cursor: 'pointer',
                        top: this.props.offset,
                        left: 0,
                        width: '100%',
                        height: '100%'
                    } }));
        }
    }

    class ColumnSeparatorRenderer extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.state = {
                deltaY: null
            };
            this.handleDragStart = (event, dragInformation) => {
                const deltaY = this.props.clamp(this.props.index, dragInformation.current.y - dragInformation.start.y);
                this.setState({ deltaY });
            };
            this.handleDragMove = (event, dragInformation) => {
                const deltaY = this.props.clamp(this.props.index, dragInformation.current.y - dragInformation.start.y);
                this.setState({ deltaY });
            };
            this.handleDragEnd = (event, dragInformation) => {
                this.setState({ deltaY: null });
                const deltaY = this.props.clamp(this.props.index, dragInformation.current.y - dragInformation.start.y);
                this.props.onMove(this.props.index, deltaY);
            };
        }
        render() {
            let top = 0;
            if (this.state.deltaY !== null) {
                top = this.state.deltaY;
            }
            return React.createElement(Draggable, { onDragStart: this.handleDragStart, onDragMove: this.handleDragMove, onDragEnd: this.handleDragEnd },
                React.createElement(this.props.floatyRenderers.columnSeparatorHandleRenderer, { offset: top }));
        }
    }

    class DropAreaRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: {
                    position: 'fixed',
                    top: this.props.dropArea.top,
                    left: this.props.dropArea.left,
                    width: this.props.dropArea.width,
                    height: this.props.dropArea.height,
                    backgroundColor: 'rgba(0, 0, 0, 0.5)'
                } });
        }
    }

    class FloatingRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: { display: 'grid', gridTemplateRows: 'minmax(min-content, max-content) 1fr' } },
                React.createElement(this.props.floatyRenderers.floatingTabRenderer, { floatyManager: this.props.floatyManager, stackItem: this.props.floating }),
                React.createElement(this.props.floatyRenderers.floatingContentRenderer, { floatyManager: this.props.floatyManager, stackItem: this.props.floating }));
        }
    }

    class LayoutRenderer extends React.PureComponent {
        render() {
            switch (this.props.layout.type) {
                case 'column':
                    return this.renderColumn(this.props.layout);
                case 'row':
                    return this.renderRow(this.props.layout);
                case 'stack':
                    return this.renderStack(this.props.layout);
            }
        }
        renderColumn(column) {
            return React.createElement(this.props.floatyRenderers.columnRenderer, { floatyRenderers: this.props.floatyRenderers, floatyManager: this.props.floatyManager, column: column });
        }
        renderRow(row) {
            return React.createElement(this.props.floatyRenderers.rowRenderer, { floatyRenderers: this.props.floatyRenderers, floatyManager: this.props.floatyManager, row: row });
        }
        renderStack(stack) {
            return React.createElement(this.props.floatyRenderers.stackRenderer, { floatyRenderers: this.props.floatyRenderers, floatyManager: this.props.floatyManager, stack: stack });
        }
    }

    class RowRenderer extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.container = null;
            this.handleRef = (ref) => {
                this.container = ref;
            };
            this.clamp = (index, deltaX) => {
                if (this.container) {
                    const widthA = this.container.children[index * 2].getBoundingClientRect().width;
                    const widthB = this.container.children[index * 2 + 2].getBoundingClientRect().width;
                    return Math.min(widthB, Math.max(-widthA, deltaX));
                }
                return Number.NaN;
            };
            this.handleMove = (index, deltaX) => {
                if (this.container) {
                    const widthA = this.container.children[index * 2].getBoundingClientRect().width;
                    const widthS = this.container.children[index * 2 + 1].getBoundingClientRect().width;
                    const widthB = this.container.children[index * 2 + 2].getBoundingClientRect().width;
                    const widthTotal = widthA + widthS + widthB;
                    const fractionA = widthA / widthTotal;
                    const fractionS = widthS / widthTotal;
                    const fractionB = widthB / widthTotal;
                    const fractionTotal = fractionA + fractionS + fractionB;
                    const deltaF = fractionTotal * deltaX / widthTotal;
                    const fractionOriginal = this.props.row.items[index].fraction + this.props.row.items[index + 1].fraction;
                    const normalizer = fractionOriginal / (fractionA + fractionB);
                    this.props.floatyManager.updateRowFractions(this.props.row, index, normalizer * (fractionA + deltaF), index + 1, normalizer * (fractionB - deltaF));
                }
            };
        }
        render() {
            const gridTemplateColumns = [];
            const elements = [];
            for (let i = 0; i < this.props.row.items.length; i++) {
                if (i > 0) {
                    gridTemplateColumns.push('max-content');
                    elements.push(React.createElement(this.props.floatyRenderers.rowSeparatorRenderer, { key: `${i}-seperator`, floatyRenderers: this.props.floatyRenderers, index: i - 1, onMove: this.handleMove, clamp: this.clamp }));
                }
                gridTemplateColumns.push(`minmax(0, ${this.props.row.items[i].fraction}fr)`);
                elements.push(React.createElement(this.props.floatyRenderers.layoutRenderer, { key: i, floatyRenderers: this.props.floatyRenderers, floatyManager: this.props.floatyManager, layout: this.props.row.items[i].child }));
            }
            return React.createElement("div", { ref: this.handleRef, style: { display: 'grid', gridTemplateColumns: gridTemplateColumns.join(' '), width: '100%', height: '100%' } }, elements);
        }
    }

    class RowSeparatorHandleRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: { position: 'relative', width: 6, height: '100%' } },
                React.createElement("div", { style: {
                        backgroundColor: 'black',
                        position: 'absolute',
                        cursor: 'pointer',
                        top: 0,
                        left: this.props.offset,
                        width: '100%',
                        height: '100%'
                    } }));
        }
    }

    class RowSeparatorRenderer extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.state = {
                deltaX: null
            };
            this.handleDragStart = (event, dragInformation) => {
                const deltaX = this.props.clamp(this.props.index, dragInformation.current.x - dragInformation.start.x);
                this.setState({ deltaX });
            };
            this.handleDragMove = (event, dragInformation) => {
                const deltaX = this.props.clamp(this.props.index, dragInformation.current.x - dragInformation.start.x);
                this.setState({ deltaX });
            };
            this.handleDragEnd = (event, dragInformation) => {
                this.setState({ deltaX: null });
                const deltaX = this.props.clamp(this.props.index, dragInformation.current.x - dragInformation.start.x);
                this.props.onMove(this.props.index, deltaX);
            };
        }
        render() {
            let left = 0;
            if (this.state.deltaX !== null) {
                left = this.state.deltaX;
            }
            return React.createElement(Draggable, { onDragStart: this.handleDragStart, onDragMove: this.handleDragMove, onDragEnd: this.handleDragEnd },
                React.createElement(this.props.floatyRenderers.rowSeparatorHandleRenderer, { offset: left }));
        }
    }

    class StackContainerRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: {
                    display: 'grid',
                    gridTemplateRows: 'max-content 1fr',
                    width: '100%',
                    height: '100%'
                } }, this.props.children);
        }
    }

    class StackRenderer extends React.PureComponent {
        constructor() {
            super(...arguments);
            this.raf = undefined;
            this.tabRefs = new Map();
            this.tabFillerRef = null;
            this.containerRef = null;
            this.tabDropAreas = new Map();
            this.tabFillerDropArea = null;
            this.containerDropArea = null;
            this.handleTabFillerRef = (element) => {
                this.tabFillerRef = element;
            };
            this.handleTabRef = (identifier) => (element) => {
                if (element) {
                    this.tabRefs.set(identifier, element);
                }
                else {
                    this.tabRefs.delete(identifier);
                }
            };
            this.updateDropAreas = () => {
                const dropResolutions = [];
                let changed = false;
                for (let i = 0; i < this.props.stack.items.length; i++) {
                    const stackItem = this.props.stack.items[i];
                    if (!this.tabRefs.has(stackItem.key)) {
                        continue;
                    }
                    const newDropArea = computeDropArea(this.tabRefs.get(stackItem.key));
                    dropResolutions.push({
                        type: 'tab',
                        dropArea: newDropArea,
                        stack: this.props.stack,
                        index: i
                    });
                    if (!this.tabDropAreas.has(stackItem.key) || !dropAreasEqual(this.tabDropAreas.get(stackItem.key), newDropArea)) {
                        changed = true;
                    }
                }
                if (this.tabFillerRef !== null) {
                    const newDropArea = computeDropArea(this.tabFillerRef);
                    dropResolutions.push({
                        type: 'tab',
                        dropArea: newDropArea,
                        stack: this.props.stack,
                        index: this.props.stack.items.length
                    });
                    if (this.tabFillerDropArea === null || !dropAreasEqual(this.tabFillerDropArea, newDropArea)) {
                        changed = true;
                    }
                }
                if (this.containerRef !== null) {
                    const newDropArea = computeDropArea(this.containerRef);
                    dropResolutions.push({
                        type: 'container',
                        dropArea: newDropArea,
                        stack: this.props.stack
                    });
                    if (this.containerDropArea === null || !dropAreasEqual(this.containerDropArea, newDropArea)) {
                        changed = true;
                    }
                }
                if (changed) {
                    this.props.floatyManager.registerDropResolutions(this, dropResolutions);
                }
                this.raf = window.requestAnimationFrame(this.updateDropAreas);
            };
        }
        componentDidMount() {
            const container = ReactDOM.findDOMNode(this);
            if (container instanceof Element) {
                this.containerRef = container;
            }
            this.updateDropAreas();
        }
        componentWillUnmount() {
            if (this.raf !== undefined) {
                window.cancelAnimationFrame(this.raf);
            }
            this.props.floatyManager.unregisterDropResolutions(this);
            this.containerRef = null;
        }
        render() {
            return React.createElement(this.props.floatyRenderers.stackContainerRenderer, { floatyManager: this.props.floatyManager, stack: this.props.stack },
                React.createElement(this.props.floatyRenderers.stackTabsRenderer, { floatyManager: this.props.floatyManager, stack: this.props.stack },
                    this.props.stack.items.map((stackItem, index) => {
                        return React.createElement("div", { key: stackItem.key, ref: this.handleTabRef(stackItem.key) },
                            React.createElement(this.props.floatyRenderers.tabRenderer, { floatyManager: this.props.floatyManager, stack: this.props.stack, stackItemIndex: index, stackItem: stackItem }));
                    }),
                    React.createElement("div", { ref: this.handleTabFillerRef },
                        React.createElement(this.props.floatyRenderers.tabFillerRenderer, { floatyManager: this.props.floatyManager, stack: this.props.stack }))),
                React.createElement(this.props.floatyRenderers.contentRenderer, { key: this.props.stack.items[this.props.stack.active].key, floatyManager: this.props.floatyManager, stack: this.props.stack, stackIndex: this.props.stack.active, stackItem: this.props.stack.items[this.props.stack.active] }));
        }
    }

    class StackTabsRenderer extends React.PureComponent {
        render() {
            return React.createElement("div", { style: { display: 'grid', gridTemplateColumns: `${this.props.stack.items.map(() => 'minmax(min-content, max-content)').join(' ')} 1fr` } }, this.props.children);
        }
    }

    class TabFillerRenderer extends React.PureComponent {
        render() {
            return null;
        }
    }



    var index = /*#__PURE__*/Object.freeze({
        ColumnRenderer: ColumnRenderer,
        ColumnSeparatorHandleRenderer: ColumnSeparatorHandleRenderer,
        ColumnSeparatorRenderer: ColumnSeparatorRenderer,
        DropAreaRenderer: DropAreaRenderer,
        FloatingRenderer: FloatingRenderer,
        LayoutRenderer: LayoutRenderer,
        RowRenderer: RowRenderer,
        RowSeparatorHandleRenderer: RowSeparatorHandleRenderer,
        RowSeparatorRenderer: RowSeparatorRenderer,
        StackContainerRenderer: StackContainerRenderer,
        StackRenderer: StackRenderer,
        StackTabsRenderer: StackTabsRenderer,
        TabFillerRenderer: TabFillerRenderer
    });

    const defaultRenderers = {
        columnRenderer: ColumnRenderer,
        columnSeparatorHandleRenderer: ColumnSeparatorHandleRenderer,
        columnSeparatorRenderer: ColumnSeparatorRenderer,
        dropAreaRenderer: DropAreaRenderer,
        floatingRenderer: FloatingRenderer,
        layoutRenderer: LayoutRenderer,
        rowRenderer: RowRenderer,
        rowSeparatorHandleRenderer: RowSeparatorHandleRenderer,
        rowSeparatorRenderer: RowSeparatorRenderer,
        stackContainerRenderer: StackContainerRenderer,
        stackRenderer: StackRenderer,
        stackTabsRenderer: StackTabsRenderer,
        tabFillerRenderer: TabFillerRenderer
    };

    exports.Floaty = Floaty;
    exports.Renderers = index;
    exports.StaticUtilities = StaticUtilities;
    exports.computeDropArea = computeDropArea;
    exports.defaultRenderers = defaultRenderers;
    exports.dropAreasEqual = dropAreasEqual;
    exports.pointInDropArea = pointInDropArea;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=bundle.js.map
